-- THE BG
application:setBackgroundColor(0x00aaee)

-- SOME VARIABLES
local myappwidth, myappheight = application:getContentWidth(), application:getContentHeight()

-- THE OBJs
local objground = loadObj("gfx/ground_grass", "ground_grass.obj")
local objmap01 = loadObj("3D_maps", "3d_map01.obj")
local objwell = loadObj("gfx/well", "well.coveredopen.obj")
local objplayer = loadObj("gfx/cubex01", "cubex01.obj")

-- THE OBJs DIMENSIONS
local map01minx, map01miny, map01minz, map01maxx, map01maxy, map01maxz =
			objmap01.min[1], objmap01.min[2], objmap01.min[3],
			objmap01.max[1], objmap01.max[2], objmap01.max[3]
local map01cx, map01cy, map01cz =
			(map01minx + map01maxx) / 2, (map01miny + map01maxy) / 2, (map01minz + map01maxz) / 2
local wellminx, wellminy, wellminz, wellmaxx, wellmaxy, wellmaxz =
			objwell.min[1], objwell.min[2], objwell.min[3],
			objwell.max[1], objwell.max[2], objwell.max[3]
local wellcx, wellcy, wellcz =
			(wellminx + wellmaxx) / 2, (wellminy + wellmaxy) / 2, (wellminz + wellmaxz) / 2
local wellwidth, wellheight, welllength =
			wellmaxx - wellminx, wellmaxy - wellminy, wellmaxz - wellminz
local playerminx, playerminy, playerminz, playermaxx, playermaxy, playermaxz =
			objplayer.min[1], objplayer.min[2], objplayer.min[3],
			objplayer.max[1], objplayer.max[2], objplayer.max[3]
local playercx, playercy, playercz =
			(playerminx + playermaxx) / 2, (playerminy + playermaxy) / 2, (playerminz + playermaxz) / 2
local playerwidth, playerheight, playerlength =
			playermaxx - playerminx, playermaxy - playerminy, playermaxz - playerminz

-- THE VIEWPORT
local mysprite = Sprite.new()
local myviewport = Viewport.new()
local mymatrix = Matrix.new()
--mymatrix:perspectiveProjection(30, -1, 1, 1000)
mymatrix:perspectiveProjection(90, -1, 1, 1000)
myviewport:setProjection(mymatrix)
myviewport:setPosition(myappwidth / 2, myappheight / 2)
myviewport:setScale(myappwidth, myappheight, 1)
myviewport:setContent(mysprite)
stage:addChild(myviewport)

-- THE LIGHT
local lx, ly, lz = myviewport:getTransform():transformPoint(1, 1, 1)
--Lighting.setLight(lx, ly, lz, 0.5)
--Lighting.setSpecular(0.3, 0.3, 0.3, 32)
Lighting.setLight(lx, ly, lz, -0.5)
Lighting.setSpecular(0.3, 0.3, 0.3, 32)

-- 1 GROUND INSTANCE
local ground1 = Viewport.new()
ground1:setContent(objground)
ground1:setScale(16, 1, 16)
ground1:setPosition(0, 0, -3)
mysprite:addChild(ground1)

-- THE MAP01
local map01 = Viewport.new()
map01:setContent(objmap01)
map01:setScale(5, 7, 5)
map01:setPosition(-8, objground.min[2] - map01miny, 40)
mysprite:addChild(map01)
Lighting.apply(objmap01)

-- SOME WELL INSTANCES
local wells_list = {}
for w = 1, 20 do
	local well = Viewport.new()
	well:setContent(objwell)
	well:setPosition(math.random(-75, 75), objground.min[2] + wellcy, math.random(-75, 75))
	well.id = w
	well.posx, well.posy, well.posz = well:getX(), well:getY(), well:getZ()
	mysprite:addChild(well)
	wells_list[w] = well
end
Lighting.apply(objwell)

-- 1 PLAYER INSTANCE
local player1 = Viewport.new()
player1:setContent(objplayer)
player1:setRotationY(270)
player1:setScale(0.5)
--player1:setPosition(0, objground.max[2] - playerminy, 4)
player1:setPosition(0, 7, 4)
mysprite:addChild(player1)
Lighting.apply(objplayer)

-- SOME DEBUGGING
local mytext = TextField.new(nil, "XXX")
mytext:setScale(3)
mytext:setPosition(0, 24)
stage:addChild(mytext)

-- A CAMERA
local camx, camy, camz, camrx, camry
local camdirx, camdirz

-- THE GAME
local game = {}
game.keys = {}
player1.speed = 0.25
player1.vy = 0 -- for gravity and jump
local iscollidewellsides = false
local oldposx, oldposy, oldposz = 0, 0, 0

function update(e)
	-- debugging
	mytext:setText("a = "..player1:getY().."\nb = "..oldposx.."\nc = ".."N/A")

	-- gravity
	player1.vy -= 0.25 * e.deltaTime
	player1:setY(player1:getY() + player1.vy)

	if player1:getRotationY() > 360 then player1:setRotationY(0) end
	if player1:getRotationY() < 0 then player1:setRotationY(360) end

	-- collisions processing
	if player1:getY() < objground.max[2] - playerminy then
		player1:setY(objground.max[2] - playerminy)
	end
	if iscollidewellsides then
		-- move the player away from the well
		player1:setX(oldposx)
		player1:setZ(oldposz - playerlength / 2)
		iscollidewellsides = false
	end

	-- collisions checking
	for w = 1, #wells_list do
		if CheckWellCollisions(player1:getX(), player1:getY(), player1:getZ(),
				playerwidth, playerheight, playerlength,
				wells_list[w].posx, wells_list[w].posy, wells_list[w].posz,
				wellwidth, wellheight, welllength) then
			iscollidewellsides = true
		end
	end

	-- 2nd setup LOOKAT
	if game.keys[KeyCode.UP] then
		player1:setX(player1:getX() + math.cos(^< player1:getRotationY()) * player1.speed)
		player1:setZ(player1:getZ() - math.sin(^< player1:getRotationY()) * player1.speed)
	end
	if game.keys[KeyCode.DOWN] then
		player1:setX(player1:getX() - math.cos(^< player1:getRotationY()) * player1.speed)
		player1:setZ(player1:getZ() + math.sin(^< player1:getRotationY()) * player1.speed)
	end
	if game.keys[KeyCode.LEFT] then
		if game.keys[KeyCode.UP] then
			player1:setRotationY(player1:getRotationY() + 1)
		elseif game.keys[KeyCode.DOWN] then
			player1:setRotationY(player1:getRotationY() - 1)
		else
			player1:setRotationY(player1:getRotationY() + 1)
		end
	end
	if game.keys[KeyCode.RIGHT] then
		if game.keys[KeyCode.UP] then
			player1:setRotationY(player1:getRotationY() - 1)
		elseif game.keys[KeyCode.DOWN] then
			player1:setRotationY(player1:getRotationY() + 1)
		else
			player1:setRotationY(player1:getRotationY() - 1)
		end
	end
	if game.keys[KeyCode.SPACE] then
		-- JUMP
		player1.vy = 0.185
		game.keys[KeyCode.SPACE] = false
	end

	camx, camy, camz, camrx, camry =
		player1:getX(), player1:getY() + 16, player1:getZ(),
		player1:getRotationX(), player1:getRotationY()
	camdirx, camdirz = math.cos(^< -camry), math.sin(^< -camry)
	-- place the camera behind the player
	camx -= camdirx * 4
	camy -= 13
	camz -= camdirz * 4
	-- FPS STYLE
	myviewport:lookAt(camx, camy, camz, player1:getX(), player1:getY() + 2, player1:getZ(),	0, 1, 0)
end

-- FUNCTIONS
function CheckWellCollisions(box1x, box1y, box1z, box1w, box1h, box1l,
			box2x, box2y, box2z, box2w, box2h, box2l)
    if box1x > box2x + box2w - 1 or -- Is box1 on the right side of box2?
       box1y > box2y + box2h - 1 or -- Is box1 under box2?
       box1z > box2z + box2l - 1 or -- Is box1 in front of box2?
       box2x > box1x + box1w - 1 or -- Is box2 on the right side of box1?
       box2y > box1y + box1h - 1 or -- Is box2 under box1?
       box2z > box1z + box1l - 1    -- Is box2 in front of box1?
    then
        return false				-- No collision.  Yay!
    else
		oldposx, oldposy, oldposz = box1x, box1y, box1z
        return true					-- Yes collision.  Ouch!
    end
end

-- KEYS HANDLER
function keyDown(event)
	local key, rc = event.keyCode, event.realCode
	game.keys[key] = true
	game.keys[rc] = true
end

function keyUp(event)
	local key, rc = event.keyCode, event.realCode
	game.keys[key] = false
	game.keys[rc] = false
end

stage:addEventListener(Event.ENTER_FRAME, update)
stage:addEventListener(Event.KEY_DOWN, keyDown)
stage:addEventListener(Event.KEY_UP, keyUp)
